define((()=>(()=>{var t={758:(t,e,n)=>{n.p=document.querySelector("body").getAttribute("data-base-url")+"nbextensions/ttgtcanvas",window.require&&window.require.config({map:{"*":{ttgtcanvas:"nbextensions/ttgtcanvas/index"}}}),t.exports={load_ipython_extension:function(){}}}},e={};function n(r){var o=e[r];if(void 0!==o)return o.exports;var i=e[r]={exports:{}};return t[r](i,i.exports,n),i.exports}return n.p="",n(758)})()));